// =====================================================
// === HOLA ============================================
//=======================================================
// Logic analyser
//=======================================================
// the controller memory block
// -- addr 0 -- new data? yes==1
// -- addr 1 -- allow capture to occur -- arm 
// -- addr 2 -- trigger count (after trigger event)
// -- addr 250 -- data capture complete == 1
// -- addr 251 -- addr_complete (sram addr of trigger sample)
// read trigger mask -- two 32 bit words 
// -- addr 254 dont care mask
// -- addr 255 actual bits
//
// The analyzer data block logs the data input at 100 MHz.
// The memory wraps at 1024 words
//
// There are three state machines running:
// (1) The data capture machine which logs one 32-bit word/cycle
//    all the time. When the trigger event occurs, it grabs a set
//    number more of samples, then signals the HPS and waits.
// (2) The trigger state machine just waits to set the trigger on EACH cycle
// (3) The control state machine gets parameters from the HPS, and when the
//    logic samples are logged, signals the HPS to dump them, then waits.

module logic_analyser(
   // the logical inputs
	data_input, data_input_export,
	ext_trigger_source, ext_trigger_source_export,
	// data memory
	sram_writedata_hps, sram_readdata_hps, sram_addr_hps, sram_write_en_hps,
	sram_clken, sram_byteenable, sram_waitrequest, sram_chipselect, sram_readdata_fpga,
	// control memory
	control_writedata_hps, control_readdata_hps, control_addr_hps, control_write_en_hps,
	control_clken, control_chipselect, control_readdata_fpga,
	// other things
	clock, reset, debug, 
	toggletest
	
);
// DUT inputs
input reset;
input wire [31:0] data_input, ext_trigger_source ;
// data memory

output  sram_byteenable ;
output sram_clken;
input sram_waitrequest ;
output  sram_chipselect ;
// control memory

output control_clken;
output control_chipselect;
input [31:0] data_input_export;
input [31:0] ext_trigger_source_export;

// the 100 MHz clock
input clock;
output [31:0] debug;
output reg toggletest;

//=======================================================
// Assign constants for SRAM Qsys ports
//=======================================================
assign control_byteenable = 4'b1111;
assign control_clken = 1;
assign control_chipselect = 1;
assign sram_byteenable = 4'b1111;
assign sram_chipselect = 1;
assign sram_clken = 1;
//=======================================================
// Controls for analyser sram slave exported in system 
//=======================================================

reg [7:0] analyser_state ;

//=======================================================
// Controls for controller sram slave exported in system 
//=======================================================
reg [7:0] control_state ;
//reg [3:0] trigger_time_mode, trigger_source_mode;
// desired number of samples after trigger
reg [9:0] trigger_count ;
// actual current count
reg [9:0] current_count ;
// setting this enables the trigger
// --set by HPS--
reg capture_arm ;
// analyser complete signal to HPS
reg complete ;
// value of data address when trigger occurs
reg [9:0] addr_complete ;

//=======================================================
// controls for analyser to/from fpga
//=======================================================
// signal to store data
wire  ext_trigger ;
reg [7:0] trigger_state ;
reg trigger ;
// trigger processing
// mask and trigger value set by HPS
reg [31:0] trig_mask, trig_value ;
assign ext_trigger = ((ext_trigger_source_export & trig_mask) == (trig_value & trig_mask))? 1'b1 : 1'b0 ;

//=======================================================
// Logic analyzer state machinedo the work outlined above
//=======================================================
// CLOCK_100 is the sdram clock from Qsys
assign debug[7:0] = sram_addr_fpga; 


//=====================================
// M10K block initialization
//=====================================


output [31:0] sram_readdata_fpga;
output [31:0] sram_readdata_hps ;
reg  [31:0] sram_writedata_fpga; 
input [31:0] sram_writedata_hps ;
reg [11:0] sram_addr_fpga, sram_write_addr_fpga;
input [11:0] sram_addr_hps; 
reg sram_write_en_fpga;
input sram_write_en_hps;

//
// First Port - HPS
// Second Port - FPGA
//
true_dual_port_ram_single_clock analyzer_buffer(
	.data_a(sram_writedata_hps), 
	.data_b(sram_writedata_fpga),
	.addr_a(sram_addr_hps), 
	.addr_b(sram_addr_fpga),
	.we_a(sram_write_en_hps),
	.we_b(sram_write_en_fpga),
	.clk(clock),
	.q_a(sram_readdata_hps), 
	.q_b(sram_readdata_fpga)
);

output [31:0] control_readdata_fpga;
wire [31:0] control_fpga_out;
assign control_readdata_fpga = control_fpga_out;

output [31:0] control_readdata_hps ;
reg  [31:0] control_writedata_fpga;
input [31:0] control_writedata_hps;
reg [11:0] control_addr_fpga;
input [11:0] control_addr_hps; 
reg control_write_en_fpga;
input control_write_en_hps;

true_dual_port_ram_single_clock control_buffer(
	.data_a(control_writedata_hps), 
	.data_b(control_writedata_fpga),// Fpga side
	.addr_a(control_addr_hps), 
	.addr_b(control_addr_fpga), // Fpga side
	.we_a(control_write_en_hps),
	.we_b(control_write_en_fpga), // Fpga side
	.clk(clock),
	.q_a(control_readdata_hps), 
	.q_b(control_fpga_out) // fpga side
);


//assign sram_writedata = data_input_export;
always @(posedge clock) begin // CLOCK_100
	//=====================================
   // reset state machine and read/write controls
	//=====================================
	if (reset) begin
		// the state machines
		analyser_state <= 8'd0 ;
		control_state <= 8'd0 ;
		trigger_state <= 8'd0 ;
		sram_write_addr_fpga <= 12'd0;
		// analyzer sram
		sram_write_en_fpga <= 1'b0 ;
		sram_addr_fpga <= 12'd0 ;
		// control sram
//		control_write_addr <= 12'd0;
		control_write_en_fpga <= 1'b0;
//		control_read_addr <= 12'd0;
		capture_arm <= 1'b0 ;
		complete <= 0 ;
		addr_complete <= 0;
		trigger <= 0;
	end
	// not reset
	else begin
	//=====================================
	// Analyser state machine
	//=====================================
	// fill a circular buffer with data 
	// once/cycle
	// until a trigger event occurs
	if (analyser_state == 8'd0) begin
		sram_writedata_fpga <= data_input_export ;
		sram_addr_fpga <= sram_write_addr_fpga;
		sram_write_en_fpga <= 1'b1 ;
		sram_write_addr_fpga <= sram_addr_fpga + 12'd1 ;
		if (trigger) begin
			trigger <= 0 ;
			analyser_state <= 8'd2;
			// record the trigger address for the 
			// HPS to use
			addr_complete <= sram_write_addr_fpga ;
			// record the trigger on the hex display
			//hex3_hex0 <= data_input ;
			// reset the post-trigger sample count 
			current_count <= 10'b0 ; // the # samples after trigger
		end
	end
	
	// must take 100, take 500 samples more, or
	// 900 samples more.
	// corresponding to trigger_count from HPS
	if (analyser_state == 8'd2) begin
		sram_writedata_fpga <= data_input_export ;
		sram_addr_fpga <= sram_write_addr_fpga;
		sram_write_en_fpga <= 1'b1 ;
		sram_write_addr_fpga <= sram_addr_fpga + 12'd1 ;
		// have we gotten the request # samples 
		// after the trigger
		if (current_count<=trigger_count) begin
			current_count <= current_count + 10'd1;	
		end
		else begin
			// allow the HPS to read the data
			// set complete flag
			analyser_state <= 8'd3;
			complete <= 1'b1 ;
			// reset for next time
			//current_count <= 12'b0;
		end	
	end
	
	// wait for HPS to read all the data points
	// then restart aqusition of data
	if (analyser_state == 8'd3) begin
		if (complete==1'b0) begin
			analyser_state <= 8'd0;
		end
	end
	
	//=====================================
	// Trigger state machine
	//=====================================
	if (trigger_state == 8'd0) begin
		if (ext_trigger==1'b1 && capture_arm ) begin
			trigger <= 8'd1;
			// record the trigger on the hex display
			// hex3_hex0 <= data_input ;
			// and clear the arming signal
			// until set again by HPS
			capture_arm <= 0 ;
		end
	end
	
	//=====================================
	// Control state machine
	//=====================================
	// Is there new data from the HPS
	// If so, read it, and set control
	// signals
	
	//=====================================
	// check for new data from HPS
	// AND send complete data, if any
	// "complete" flag is set in analyser state machine
	if (control_state == 8'd0) begin	
		if (complete) begin
			control_state <= 8'd200 ;
			control_write_en_fpga <= 1'b0 ;
		end
		// get HPS signal for new data
		else begin
			control_addr_fpga <= 8'd0 ;
			control_write_en_fpga <= 1'b0 ;
			control_state <= 8'd1 ;
		end
	end
	
	// wait
	if (control_state == 8'd1) begin
		control_state <= 8'd2 ;
	end
	
	// read determine if there is new data
	if (control_state == 8'd2) begin
		if (control_readdata_fpga > 0) begin
			// get the new data
			control_state <= 8'd7 ;
		end
		else begin
			control_state <= 8'd0 ;
		end
	end
	//=====================================
	
	// === read the arm command ===========
	// do this LAST !!!
	if (control_state == 8'd4) begin
		control_addr_fpga <= 10'd1 ;
		control_write_en_fpga <= 1'b0 ;
		control_state <= 8'd5 ;
	end
	
	// wait
	if (control_state == 8'd5) begin
		control_state <= 8'd6 ;
	end
	
	// read arm_control
	if (control_state == 8'd6) begin
		capture_arm <= control_readdata_fpga ;
		control_state <= 8'd100 ;
	end
	//=====================================
	
	// === read the trigger time mode =====
	if (control_state == 8'd7) begin
		control_addr_fpga <= 10'd2 ;
		control_write_en_fpga <= 1'b0 ;
		control_state <= 8'd8 ;
	end
	
	// wait
	if (control_state == 8'd8) begin
		control_state <= 8'd9 ;
	end
	
	// read trigger_count
	// number of samples AFTER trigger
	if (control_state == 8'd9) begin
		trigger_count <= control_readdata_fpga ;
		control_state <= 8'd10 ;
	end
	
	// === read the trigger mask  ========
	if (control_state == 8'd10) begin
		control_addr_fpga <= 10'd254 ;
		control_write_en_fpga <= 1'b0 ;
		control_state <= 8'd11 ;
	end
	
	// wait
	if (control_state == 8'd11) begin
		control_state <= 8'd12 ;
	end
	
	// read trigger_mask
	if (control_state == 8'd12) begin
		trig_mask <= control_readdata_fpga ;
		control_state <= 8'd13 ;
	end
	
	// === read the trigger value  ======
	if (control_state == 8'd13) begin
		control_addr_fpga <= 10'd255 ;
		control_write_en_fpga <= 1'b0 ;
		control_state <= 8'd14 ;
	end
	
	// wait
	if (control_state == 8'd14) begin
		control_state <= 8'd15 ;
	end
	
	// read trigger_value
	if (control_state == 8'd15) begin
		trig_value <= control_readdata_fpga ;
		control_state <= 8'd4 ;
	end
	
	//=====================================
	
	// === clear the complete-flag 
	// should be after the data readback
	//if (control_state == 8'd10) begin
		//control_addr_fpga <= 8'd250 ;
		//control_writedata_fpga <= 1'b0;
		//control_write_en_fpga <= 1'b1 ;
		//control_state <= 8'd100 ;
	//end
	
	// === clear the "new data" flag ======
	// === and return to idle state
	if (control_state == 8'd100) begin
		control_addr_fpga <= 10'd0 ;
		control_writedata_fpga <= 1'b0;
		control_write_en_fpga <= 1'b1 ;
		control_state <= 8'd0 ;
	end
	//=====================================
	
	// === send the addr_complete value to HPS
	// and send done command
	if (control_state == 8'd200) begin
		control_addr_fpga <= 10'd251 ;
		control_writedata_fpga <= addr_complete ;
		control_write_en_fpga <= 1'b1 ;
		control_state <= 8'd201 ;
	end
	// now send done/complete signal
	// back to HPS
	if (control_state == 8'd201) begin
		control_addr_fpga <= 10'd250 ;
		control_writedata_fpga <= 1'b1 ;
		control_write_en_fpga <= 1'b1 ;
		control_state <= 8'd202 ;
	end
	// and wait for HPS rsponse
	// read back HPS done with data readback
	if (control_state == 8'd202) begin
		control_addr_fpga <= 10'd250 ;
		control_write_en_fpga <= 1'b0 ;
		control_state <= 8'd203 ;
	end
	// wait
	if (control_state == 8'd203) begin
		control_state <= 8'd204 ;
	end
	// wait
	if (control_state == 8'd204) begin
		if (control_readdata_fpga==0) begin
			// and clear the flag
			//complete <= 1'b0 ;
			control_state <= 8'd0 ;
			complete <= 1'b0 ;
		end
		else begin // try again
			control_state <= 8'd202 ;
		end
	end
	end // reset else
end // always @(posedge state_clock)

endmodule // module logic_analyser
/// end /////////////////////////////////////////////////////////////////////

module true_dual_port_ram_single_clock
#(parameter DATA_WIDTH = 32, ADDR_WIDTH = 12)
(
	input [(DATA_WIDTH-1):0] data_a, data_b,
	input [(ADDR_WIDTH-1):0] addr_a, addr_b,
	input we_a, we_b, clk,
	output reg [(DATA_WIDTH-1):0] q_a, q_b
);


	// Declare the RAM variable
	reg [DATA_WIDTH-1:0] ram[2**ADDR_WIDTH-1:0] /* synthesis ramstyle = "no_rw_check, M10K" */ ; 

	always @ (posedge clk)
	begin // Port a
		if (we_a)
		begin
			ram[addr_a] <= data_a;
			q_a <= data_a;
		end
		else
			q_a <= ram[addr_a];
	end
	always @ (posedge clk)
	begin // Port b
		if (we_b)
		begin
			ram[addr_b] <= data_b;
			q_b <= data_b;
		end
		else
			q_b <= ram[addr_b];
	end
endmodule
